/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCharacterData.h"
#include "gdom/GdomText.h"
#include "gdom/PrivateGdomCharacterData.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomText.h"
#include "gdom/PrivateIntGdomText.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMText::GDOMText(PassRefPtr<WebCore::Text> impl)
    : GDOMCharacterData(impl)
{
}

GdomText *
gdom_text_split_text (GdomText *thiz, gulong offset)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Text * item = (gdom_text_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Text> g_res = WTF::getPtr(item->splitText(offset, ec));
    GdomText * res = static_cast<GdomText *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomText *
gdom_text_replace_whole_text (GdomText *thiz, gchar * content)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Text * item = (gdom_text_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (content, 0);
    WebCore::String _g_content = WebCore::String::fromUTF8(content);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Text> g_res = WTF::getPtr(item->replaceWholeText(_g_content, ec));
    GdomText * res = static_cast<GdomText *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Text.h"
#include <gdom/GdomText.h>
#include <gdom/PrivateGdomText.h>
#include <gdom/PrivateIntGdomText.h>
#include <gdom/GdomTextprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomText, gdom_text, GDOM_TYPE_CHARACTER_DATA);

/*
struct _GdomTextPrivate {
    class WebCore::Text *item;
};

#define GDOM_TEXT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_TEXT, GdomTextPrivate))
*/
G_END_DECLS
WebCore::Text *
gdom_text_get_private(GdomText *request)
{
    GDOMText *res;
    res = static_cast<GDOMText*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Text*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_WHOLE_TEXT,
};


static void gdom_text_finalize(GObject* object)
{
    /*GdomText* request = GDOM_TEXT(object);*/
    /*GdomTextPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_CHARACTER_DATA));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_text_parent_class)->finalize(object);
}

static void gdom_text__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomText *self = GDOM_TEXT(object);
    WebCore::Text * gself = (gdom_text_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_text__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomText *self = GDOM_TEXT(object);
    WebCore::Text * gself = (gdom_text_get_private(self));
    switch (prop_id) {
    case PROP_WHOLE_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->wholeText()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_text_class_init(GdomTextClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_text_finalize;
    g_klass->set_property = gdom_text__set_property_fn;
    g_klass->get_property = gdom_text__get_property_fn;

    g_object_class_install_property (g_klass, PROP_WHOLE_TEXT,
            g_param_spec_string ("whole_text", /* name */
                "text_whole_text", /* short description */
                "read-only  gchar * Text.whole_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomTextPrivate));*/
}

static void gdom_text_init(GdomText* request)
{
    /*GdomTextPrivate* priv = GDOM_TEXT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

